package com.company;


import java.util.Scanner;

public class Players {
    // the number of possible players saves here
    private int possibleMoves;

    // the player's number of disks
    private int numberOfDisks;

    /**
     * Creates a player and initials the possible moves with zero
     */
    public Players(){
        possibleMoves = 0;
    }

    /**
     * gets an array(player1 or player2) and a coordinate and checks
     * if the coordinate contains in the array
     * @param arr the array to check it's values
     * @param x the x of the coordinate
     * @param y the y of the coordinate
     * @return if contains returns true and otherwise returns false
     */
    public boolean containsInArray(int[][] arr, int x, int y){
        if (arr[x][y] != 0)
            return true;
        else
            return false;
    }

    /**
     * displays the place of black and white discs and blanks (will be overridden in console and graphic version)
     * @param a1 array of the player1's discs
     * @param a2 array of the player2's discs
     */
    public void display(int[][] a1, int[][] a2) {
        System.out.println("  A B C D E F G H");
        for (int i = 1; i <= 8; i++) {
            System.out.print(i + " ");
            for (int j = 1; j <= 8; j++) {
                if (a2[j][i] != 0) {
                    // the unicode of white circle (there is a player2 disc)
                    System.out.print('\u25CB');
                }
                else {
                    if (a1[j][i] != 0) {
                        // the unicode of black circle (there is a player1 disc)
                        System.out.print('\u25CF');
                    }
                    else
                        // the unicode of white square (a blank)
                        System.out.print('\u25A1');
                }
                System.out.print(" ");
            }
            System.out.println();
        }
    }

    /**
     * this method gets the coordinate of a blank space an checks if there is
     * a opponent disc,and then if exists for each of opponent disks does the equalize method
     * @param arrOpponent list of opponent discs' coordinates
     * @param arr list of the player disc's coordinates
     * @param x the x of the blank space
     * @param y the y of the blank space
     */
    public void searchInNeighbors(int[][] arrOpponent, int[][] arr, int x, int y){
        if (x != 1){
            if (containsInArray(arrOpponent, x - 1, y)){
                equalize(arrOpponent, arr, x, y, -1, 0);
            }
            if (y != 1){
                if (containsInArray(arrOpponent, x - 1, y - 1))
                    equalize(arrOpponent, arr, x, y, -1, -1);
            }
            if (y != 8){
                if (containsInArray(arrOpponent, x - 1, y + 1))
                    equalize(arrOpponent, arr, x, y, -1, +1);
            }
        }

        if (x != 8){
            if (containsInArray(arrOpponent, x + 1, y)){
                equalize(arrOpponent, arr, x, y, +1, 0);
            }
            if (y != 1){
                if (containsInArray(arrOpponent, x + 1, y - 1))
                    equalize(arrOpponent, arr, x, y, +1, -1);
            }
            if (y != 8){
                if (containsInArray(arrOpponent, x + 1, y + 1))
                    equalize(arrOpponent, arr, x, y, +1, +1);
            }
        }

        if (y != 1){
            if (containsInArray(arrOpponent, x, y - 1))
                equalize(arrOpponent, arr, x, y, 0, -1);
        }

        if (y != 8){
            if (containsInArray(arrOpponent, x, y + 1))
                equalize(arrOpponent, arr, x, y, 0, +1);
        }
    }



    /**
     * this method checks if the move is possible, then equalize the color of the
     * player's disc in a line from the blank to the first player's disk that seen
     * @param arrOpponent list of opponent discs' coordinates
     * @param arr list of the player disc's coordinates
     * @param x the x of the blank space
     * @param y the y of the blank space
     * @param moveX the direction to the x es
     * @param moveY the direction to the y es
     */
    public void equalize(int[][] arrOpponent, int[][]arr, int x, int y, int moveX, int moveY) {
        if (checkTheMoveIfPossible(arrOpponent, arr, x, y, moveX, moveY)) {
            arr[x][y] = 1;
            while (((x >= 1) && (x <= 8) && (y >= 1) && (y <= 8))
                    && (!containsInArray(arr, x + moveX, y + moveY))) {
                x += moveX;
                y += moveY;
                arr[x][y] = 1;
                arrOpponent[x][y] = 0;
            }
        }
    }

    /**
     * this method returns the number of possible moves by finding blanks and
     * checking sides that if there is a possible move
     * @param arrOpponent the coordinates of opponent player's discs
     * @param arr the coordinates of the player's discs
     * @return if there is a possible move returns 1 and else returns 0
     */
    public int numberOfPlayerPossibleMoves(int[][] arrOpponent, int[][]arr){
        for (int i = 1; i < 9; i++){
            for (int j = 1; j < 9; j++){
                if ((!containsInArray(arr, j, i)) && (!containsInArray(arrOpponent, j, i))){
                    //System.out.print(j);
                    //System.out.println(i);
                    if (checkSides(arrOpponent, arr, j, i)) {
                        //System.out.println("x ="+j+"y ="+i);
                        return 1;
                    }
                }
                /*else {
                    if (containsInArray(arr, j, i))
                        System.out.println("khoditrue");
                    else
                        System.out.println("khodifalse");
                    if (containsInArray(arrOpponent, j, i))
                        System.out.println("ghkhoditrue");
                    else
                        System.out.println("ghkhodifalse");
                }*/
            }
        }
        return possibleMoves;
    }

    /**
     * checks the sides of the blank to see if there is an opponent disk or not
     * and then goes to the "checkTheMoveIfPossible" method
     * @param arrOpponent the coordinates of opponent's discs
     * @param arr the coordinates of player's discs
     * @param x the x coordinate of blank
     * @param y the x coordinate of blank
     * @return true if any of sides is allowed and false if none them
     */
    public boolean checkSides(int[][] arrOpponent, int[][] arr, int x, int y){
        if (x != 1){
            if (containsInArray(arrOpponent, x - 1, y)){
                if (checkTheMoveIfPossible(arrOpponent, arr, x, y, -1, 0))
                    return true;
            }
            if (y != 1){
                if (containsInArray(arrOpponent, x - 1, y - 1))
                  if (checkTheMoveIfPossible(arrOpponent, arr, x, y, -1, -1))
                      return true;
            }
            if (y != 8){
                if (containsInArray(arrOpponent, x - 1, y + 1))
                    if (checkTheMoveIfPossible(arrOpponent, arr, x, y, -1, +1))
                        return true;
            }
        }
        if (x != 8){
            if (containsInArray(arrOpponent, x + 1, y)){
                if (checkTheMoveIfPossible(arrOpponent, arr, x, y, +1, 0))
                    return true;
            }
            if (y != 1){
                if (containsInArray(arrOpponent, x + 1, y - 1))
                    if (checkTheMoveIfPossible(arrOpponent, arr, x, y, +1, -1))
                        return true;
            }
            if (y != 8){
                if (containsInArray(arrOpponent, x + 1, y + 1)) {
                    if (checkTheMoveIfPossible(arrOpponent, arr, x, y, +1, +1))
                        return true;
                }
            }
        }
        if (y != 1){
            if (containsInArray(arrOpponent, x, y - 1))
                if (checkTheMoveIfPossible(arrOpponent, arr, x, y, 0, -1))
                    return true;
        }
        if (y != 8){
            if (containsInArray(arrOpponent, x, y + 1)) {

                if (checkTheMoveIfPossible(arrOpponent, arr, x, y, 0, +1))
                    return true;
            }
        }
        return false;
    }

    /**
     * checks the move if possible :))
     * @param arrOpponent the coordinates of opponent's discs
     * @param arr the coordinates of player's discs
     * @param x the x coordinate of blank
     * @param y the x coordinate of blank
     * @param moveX the direction in the x axis (0, 1 or -1)
     * @param moveY the direction in the x axis (0, 1 or -1)
     * @return if there is a possible move returns true and else returns false
     */
    public boolean checkTheMoveIfPossible(int[][] arrOpponent, int[][]arr, int x, int y, int moveX, int moveY){
        int x1 = x; int y1 = y;
        x1 += moveX;
        y1 += moveY;
        while ((x1 > 0) && (x1 < 9) && (y1 > 0) && (y1 < 9)){
            if (containsInArray(arrOpponent, x1, y1)){
                x1 += moveX;
                y1 += moveY;
            }
            else{
                if (containsInArray(arr, x1, y1))
                    return true;
                else
                    break;
            }
        }
        return false;
    }

    /**
     * gets a player's array and returns the number of it's discs
     * @param arr player1(2)'s array
     * @return the number of discs
     */
    public int getNumberOfDisks(int[][] arr) {
        numberOfDisks = 0;
        for (int i = 1; i <= 8; i++){
            for (int j = 1; j <= 8; j++){
                if (arr[j][i] != 0) {
                    //System.out.println("x = " + j + "y = " + i);
                    numberOfDisks++;
                }
            }
        }
        //System.out.println(numberOfDisks1);
        return numberOfDisks;
    }

}
